from enum import Enum


class Settings(Enum):
    HDSEMG_SELECT_INSTALLED = "HDSEMG_SELECT_INSTALLED"
    WORKFOLDER_PATH = "WORKFOLDER_PATH"
    OPENHDEMG_INSTALLED = "OPENHDEMG_INSTALLED"
    LOG_LEVEL = "LOG_LEVEL"


